#include "generate_signal.h"
#include "decode.h"
#include "helpers.h"
#include <fstream>
#include <iostream>
#include <chrono>

using namespace std;


int main(int argc, const char * argv[]) {

    ifstream message_file ("parameters.config");

    string placeholder; //exists just to parse file

    message_file >> placeholder;
    double SNR_dB;
    message_file >> SNR_dB;

    message_file >> placeholder;
    unsigned long sample_rate;
    message_file >> sample_rate;

    message_file >> placeholder;
    unsigned long carrier_freq;
    message_file >> carrier_freq;

    message_file >> placeholder;
    unsigned long samples_per_bit;
    message_file >> samples_per_bit;

    message_file >> placeholder;
    long periods_per_symbol;
    message_file >> periods_per_symbol;

    message_file >> placeholder;
    long overlapping_samples;
    message_file >> overlapping_samples;

    //input file is message.txt, output file is signal.txt
    chrono::time_point<std::chrono::system_clock> start_time, end_time, generate_time, decode_time; // Start and end times
    start_time = chrono::system_clock::now();
    generate_signal("message.txt", "signal.txt", sample_rate, carrier_freq, samples_per_bit, periods_per_symbol, overlapping_samples, SNR_dB);
    generate_time = chrono::system_clock::now();
    cout << "\n----- Matched Filter Timing -----" << "\n\n";
    decode("signal.txt", "decoded_message.txt", sample_rate, carrier_freq, samples_per_bit, periods_per_symbol, overlapping_samples);
    decode_time = chrono::system_clock::now();
    cout << "----- Error Rate Statistics -----" << "\n\n";
    compute_error_rates("message.txt", "decoded_message.txt", SNR_dB);
    end_time = chrono::system_clock::now();
    cout << "\n----- Program Time Statistics -----" << "\n\n";
    cout << "Total time elapsed : " << chrono::duration<double>(end_time - start_time).count() << "s\n";
    cout << "Time to generate signal : " << chrono::duration<double>(generate_time - start_time).count() << "s\n";
    cout << "Time to decode signal : " << chrono::duration<double>(decode_time - generate_time).count() << "s\n";
    cout << "Time to compute error rates : " << chrono::duration<double>(end_time - decode_time).count() << "s\n\n";
    cout << "-----------------------------------" << "\n\n";

    return 0;
}
